function [newPars, MagC] = Level1bVFMEstimateTCF(CCDB, oldPars, ...
                                                 VFM, dB, iVFM, VFMraw, ...
                                                 ASM);
% [newPars, MagC] =
%   Level1bVFMEstimateTCF(CCDB, oldPars, VFM, dB, iVFM, VFMraw, ASM)
%
% Estimate the VFM temporal calibration parameters, scales, biases,
% and non-othogonalities, for the provided VFM (Level 1b_{Inst} vector)
% and ASM (Level 1b scalar, fully corrected) data.
%
% The processing is described in SW-RS-DSC-SY-0002
% "Level 1b Processor Algorithms".
% The CCDB argument is thoroughly described in SW-IF-DSC-SY-0004
% "Level 1b Processor Interfaces to the EESS".
%
% Input:
%   CCDB	Structure containg the Characterisation and
%		Calibration Data Base parameters
%
%   oldPars	A structure containing the input TCF.VFM
%		parameters used as apriori information
%		See newPars description below.
%
%   VFM		Structure containing Level 1b_{Inst} VFM data
%	t	    Time-instants, t_{out,VFM}, nx1 vector
%	B	    Vector measurements, CCDB calibrated. nx3 array
%	B_error	    Error estimates of B, nx3 array
%
%   dB		Structure containing the magnetic stray field
%               vectors for the VFM sensor.
%	dB	    Stray field vectors, nx3 array
%	dB_err	    Uncertainties in dB dua to missing telemetry.
%		    nx3 array
%
%   iVFM	mx1 vector containing indices into B and dB of
%		samples corresponding to F
%
%   VFMraw	Raw VFM measurements (for MagC output), nx3 array
%
%   ASM		Structure containing resampled Level 1b ASM data.
%	F	    ASM measurement, mx1 vector
%	F_error	    Error estimates of F, mx1 vector
%
% Output
%   newPars	Structure containing:
%	scale	    Vector of 3 scale parameters
%	bias	    Vector of 3 offset parameters
%	n_orth	    Vector of 3 non-orthogonality angles, arc-sec
%	trange	    Time-range of data for which the parameters
%		    are obtained
%	rms	    Weighted rms misfit of data after estimation
%
%   MagC	Structure containing Level 1b Mag-C (magnetic
%		calibration data), see SW-RS-DSC-SY-0002
%
%
% $Revision: 2.2 $

% $Id: Level1bVFMEstimateTCF.m,v 2.2 2007/03/01 11:57:46 lastec Exp lastec $

% Start by initializing newPars - in case no data are available or
% selected.
newPars = oldPars;

% Start filling MagC
MagC.t = VFM.t(iVFM) - CCDB.ASM.Filter.Delay * 2;
MagC.F = ASM.F;
MagC.F_error = ASM.F_error;

% Verify we have sufficient number of samples for parameter estimation
if(numel(iVFM) > 250)

  % - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  % Estimate newPars parameters, m
  % - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
  
  % Some constants
  Max_iter = 30;
  step = .8;
  as2r = pi / 180 / 3600;	% arc-sec to radians
  gam = [repmat(1,3,1); repmat(3e4,3,1); repmat(3e4,3,1)];

  m_apriori  = [oldPars.bias'; ...
                oldPars.scale'; ...
                as2r * oldPars.n_orth'];
  % Make weighting of a priori information dependend on "age" of oldPars
  % - if 5 days old, weigh by half compared to most recent TCF's
  divider = (5 * 86400) / (exp(1) - 1);
  m_weights = numel(iVFM) / ...
	      (1 + log((abs(oldPars.trange(2)-VFM.t(iVFM(1))) + divider) / ...
                       divider)) * ...
	      gam .* ...
	      reshape(CCDB.VFM.EstWeights(ceil((1:9)/3)), 9, 1);
  iC_apriori = diag(m_weights);
  
  % Application of ASM filter needs to be done in gap-free batches
  batches = [0; find(diff(VFM.t) > 1.5/50); numel(VFM.t)];
  
  m = m_apriori;
  w = ones(size(iVFM));
  B = zeros(size(VFM.B));		% Allocate B
  F_B = B(:,1);				%   F_B
  Btmp = B;				%   Btmp
  wG = zeros(numel(iVFM), numel(m));	%   wG
  bigG = zeros(size(B,1), numel(m));	%   bigG
  
  for iter = 1:Max_iter		% Do the non-linear least-squares iteration

    % Start with sensitivity matrix S1 = S^{-1} and 
    % non-orthogonality matrix P1 = P^{-1} and its
    % partial derivatives dP1(:,:,k) = dP1/dm_{k+6}
    S1 = diag(1 ./ m(4:6));
    cosi = cos(m(7:9));
    sine = sin(m(7:9));
    wP = sqrt(1 - sine(2)^2 - sine(3)^3);	% Avoid clash with weights!
    w3c7 = wP^3 * cosi(1);
    P1 = [[1                0          0]; ...
          [sine(1)/cosi(1)  1/cosi(1)  0]; ...
          [-(sine(1)*sine(3) + cosi(1)*sine(2)) / (wP*cosi(1)) ...
             -sine(3) / (wP*cosi(1)) ...
               1/wP]];
    dP1(:,:,1) = ...
        [[0 0 0]; ...
         [1/cosi(1)^2  sine(1)/cosi(1)^2  0]; ...
         [-sine(3)/(wP*cosi(1)^2) ...
            -sine(1)*sine(3)/(wP*cosi(1)^2) ...
              0]];
    dP1(:,:,2) = ...
        [[0 0 0]; ...
         [0 0 0]; ...
         [-cosi(2)*(cosi(1)*cosi(3)^2 + prod(sine))/w3c7 ...
            -sine(2)*cosi(2)*sine(3)/w3c7 ...
              sine(2)*cosi(2)/wP^3]];
    dP1(:,:,3) = ...
        [[0 0 0]; ...
         [0 0 0]; ...
         [-cosi(3)*(sine(1)*cosi(2)^2 + cosi(1)*sine(2)*sine(3))/w3c7 ...
            -cosi(2)^2*cosi(3)/w3c7 ...
              sine(3)*cosi(3)/wP^3]];

    % Save some computation time - apply scale and bias by hand
    % (S1 is a diagonal matrix)
    for i=1:3
      Btmp(:,i) = S1(i,i) * (VFM.B(:,i) - m(i));
    end%for i
    B = Btmp * P1' - dB.dB;	% Complete calibration
    
    % Apply ASM filter in batches to compute:
    %	F_B = filter(|B(m)|)
    %	bigG - data kernel matrix, composed of the filtered,
    %		partial derivatives
    for i=1:numel(batches)-1
      j = batches(i)+1:batches(i+1);	% For convenience
      f = sqrt(sum(B(j,:) .^ 2, 2));	% |B| - unfiltered
      F_B(j) = ASMFilter(CCDB.ASM.Filter, f, 1/50, 2);

      % Compute bigG
      P1tB = B(j,:) * P1;	% = (P^-1)' B for j rows
      for k=1:3
        gk = -2 * S1(k,k) * P1tB(:,k) ./ f;
        bigG(j,k) = ASMFilter(CCDB.ASM.Filter, gk, 1/50, 2);
        bigG(j,k+3) = ASMFilter(CCDB.ASM.Filter, ...
                                Btmp(j,k) .* gk, ...
                                1/50, 2);
        bigG(j,k+6) = ...
            ASMFilter(CCDB.ASM.Filter, ...
                      (sum(B(j,:) .* (Btmp(j,:) * dP1(:,:,k)'), 2) + ...
                       sum(Btmp(j,:) .* (B(j,:) * dP1(:,:,k)), 2)) ./ ...
                      (2*f), ...
                      1/50, 2);
      end%for i
    end%for i

    delta_d = ASM.F - F_B(iVFM);
    rms = sqrt(sum((w .* delta_d) .^ 2) / sum(w));
    w = huber_weights(delta_d ./ rms); % robust weights

    for i=1:numel(m);	% Multiply matrix G with weights w
      wG(:,i) = w .* bigG(iVFM,i);
    end
    delta_m = (bigG(iVFM,:)' * wG + iC_apriori) \ ...
              (wG' * delta_d + iC_apriori*(m_apriori - m));
    m = m + step*delta_m;

    if(norm(gam .* delta_m) < 1e-3)
      break;		% Converged - exit loop
    end%if
  end%for iter

  % Save result
  newPars.bias    = m(1:3)';
  newPars.scale   = m(4:6)';
  newPars.n_orth  = m(7:9)' / as2r;
  newPars.rms     = rms;
  newPars.trange  = VFM.t(iVFM([1 end]))';
  newPars.samples = numel(iVFM);
  newPars.w_scale = diag(iC_apriori)';
  newPars.cov     = ...
      inv(bigG(iVFM,:)' * bigG(iVFM,:) + iC_apriori);	% Covariance matrix
  newPars.cov(:,7:9) = newPars.cov(:,7:9) / as2r;	% .. convert non-
  newPars.cov(7:9,:) = newPars.cov(7:9,:) / as2r;	% orth. elem. to as
  % d_m = sqrt(diag(newPars.cov));	% error estimates of model parameters
  
  MagC.F_VFM = F_B(iVFM);
  % Locate VFM samples nearest MagC.t - there are no gaps, hence it
  % is expected to be easy - start looking n samples before iVFM(i)
  % First, allocate spaces
  MagC.dt = zeros(numel(iVFM), 1);
  MagC.B  = zeros(numel(iVFM), 3);
  MagC.dB = zeros(numel(iVFM), 3);
  MagC.EU_VFM = zeros(numel(iVFM), 3);
  n = floor(CCDB.ASM.Filter.Delay * 2 * 50) - 10;
  for i=1:numel(iVFM)
    j = iVFM(i) - n;
    while((j > 1) && (VFM.t(j) - MagC.t(i) > MagC.t(i) - VFM.t(j-1)))
      j = j-1;
    end%while
    MagC.dt(i) = VFM.t(j) - MagC.t(i);
    MagC.B(i,:) = B(j,:);
    MagC.dB(i,:) = dB.dB(j,:);
    MagC.EU_VFM(i,:) = VFMraw(j,:);
  end%for i
end%if

return;
