% Test driver for VFM-ASM in-flight calibration test and verification
% $Id: driver.m,v 1.1 2007/12/13 09:28:20 lastec Exp $

if(exist('CalManeuver', 'var') == 0)
  CalManeuver = 0;
end%if

dbstop if error
addpath('../../Level1b')

% Load CHAMP data
load champ.mat;		% Loads: t_r lat lon alt Fgfz t B

% Initialize required part of CCDB
% The VFM.EstWeights parameters are set to zero to allow free determination
% Therefore TCF.VFM^input has no significance
CCDB.VFM.EstWeights = zeros(1,3);
CCDB.ASM.Filter.Delay = 1.125;
[CCDB.ASM.Filter.Coeff(1,:) CCDB.ASM.Filter.Coeff(2,:)] = ...
    butter(2, 0.2/25);

% Setup TCF.VFM^input structure - ideal, 1 year "old" (irrelevant
% as no apriori information is used {CCDB.VFM.EstWeights == 0})
TCF_VFM_input = struct('scale', ones(1,3), ...
                       'bias', zeros(1,3), ...
                       'n_orth', zeros(1,3), ...
                       'trange', -365 * 86400 + [0 86399]);

% Generate master VFM indices and corresponding, filtered ASM data
i_gaps = [0 find(diff(t) > 0.03) numel(t)];
iVFM_master = [];
ASMall.F = [];
for i = 2:numel(i_gaps)
  % Start indices 10 seconds after gap - if at least 15 seconds of data
  if(i_gaps(i-1) + 15*50 < i_gaps(i))
    iVFM_master = [iVFM_master; (i_gaps(i-1)+10*50:50:i_gaps(i))'];

    F_filt = ASMFilter(CCDB.ASM.Filter, ...
                       sqrt(sum(B(i_gaps(i-1)+1:i_gaps(i), :).^2, 2)), ...
                       0.02, ...
                       2);
    ASMall.F = [ASMall.F; F_filt(10*50:50:end)];
  end%if
end%for i
  
% Add 0.25 nT white noise to ASM data, set uncertainty estimate to 0.3 nT
rand('state', 79276);	% Select and seed ("SWARM") random generator
ASMall.F = ASMall.F + rand(numel(ASMall.F), 1)*0.5 - 0.25;
ASMall.F_error = repmat(0.3, size(ASMall.F));


% Models of VFM. Parameters: biases, scales, non-orthogonalities (")
VFM_Models = [...
    [ 0.00  0.10 -0.10  1.000000 0.999980 1.000020   0.0  2.0 -2.0]; ...
    [ 0.00 -1.00  1.00  1.000000 0.999980 1.000020   2.0 -2.0  0.0]; ...
    [ 1.50  0.00 -1.50  0.999950 1.000050 1.000000  -2.0  0.0  2.0]; ...
    [ 2.25 -2.00  1.50  1.000050 0.999925 1.000025   3.5  1.0  2.5]; ...
    [-3.00  2.50  2.75  1.000125 0.999850 0.999900   4.7 -3.5  4.2]; ...
    [ 6.50 -7.25  9.25  0.999800 1.000150 1.000250   8.5  6.7 -7.8]; ...
    [12.25 14.00 -9.95  1.000133 1.000167 0.999850  13.0 -9.5 11.5]];

n = 1;		% Counter for "Results" (array of results)

% Test batch lengths (hours): 3, 6, 9, 12, 18, 24
for batch=[3 6 9 12 18 24]
  % Last iVFM_master index and number of VFM 50 Hz samples in batch
  iVFM_master_max = floor(batch / 24 * numel(iVFM_master));
  n_batch = iVFM_master(iVFM_master_max);
  
  if(CalManeuver)
    % Swap axis 1 and 2 in middle half of VFM samples
    i_swap = ceil(n_batch/4):floor(n_batch*3/4);
    B(i_swap, [1 2]) = B(i_swap, [2 1]);
  end%if
  
  for model=1:size(VFM_Models,1)
    % Construct VFMraw - simulated VFM response (no temperature effects...)
    sin_u = sind(VFM_Models(model, 7:9) / 3600);
    cos_u = cosd(VFM_Models(model, 7:9) / 3600);
    P = [[1 0 0]; ...
         [-sin_u(1) cos_u(1) 0]; ...
         [ sin_u(2) sin_u(3) sqrt(1 - sin_u(2)^2 - sin_u(3)^2)]];
    VFMraw.B = B(1:n_batch,:) * P';
    for i=1:3
      VFMraw.B(:,i) = ...
          VFM_Models(model, i+3) * VFMraw.B(:,i) + VFM_Models(model, i);
    end%for i
    VFMraw.t = t(1:n_batch);
  
    % Stray fields (none)
    dB.dB = zeros(size(VFMraw.B));
    dB.dB_err = ones(size(VFMraw.B));	% Not used

    % Test different sample selection frequencies (1/Hz): 1, 2, 4, 8, 16
    for samp_freq=[1 2 4 8 16]
      i = 1:samp_freq:iVFM_master_max;
      iVFM = iVFM_master(i);
      ASM = struct('F', ASMall.F(i), 'F_error', ASMall.F_error(i));
      estPars = Level1bVFMEstimateTCF(CCDB, TCF_VFM_input, ...
                                      VFMraw, dB, iVFM, VFMraw.B, ...
                                      ASM);
      parm_err_est = sqrt(diag(estPars.cov));
      format long
      format compact
      model, batch, samp_freq, estPars, estPars.cov, parm_err_est
      Results(n) = struct('model_no', model, ...
                          'batch', batch, ...
                          'samp_freq', samp_freq, ...
                          'model', VFM_Models(model,:), ...
                          'est_pars', estPars);
      n = n + 1;
    end%for samp_freq
  end%for model
  
  if(CalManeuver)
    % Swap axis 1 and 2 back
    B(i_swap, [1 2]) = B(i_swap, [2 1]);
  end%if
end%for batch

if(CalManeuver)
  save results_cal_maneuver Results
else
  save results Results
end%if